@@ -1,5 +1,5 @@
 from strong_opx.providers.compute import ComputeInstance, ComputeInstanceDescription
-from strong_opx.providers.container_registry import current_container_registry
 from strong_opx.providers.discovery import current_provider_class, current_provider_error_handler, select_provider
+from strong_opx.providers.docker_registry import current_docker_registry
 from strong_opx.providers.provider import Provider
 from strong_opx.providers.secret_provider import SecretProvider
