@@ -14,6 +14,7 @@ VAR_NAME_RE = re.compile(r"^[a-zA-Z_][a-zA-Z_0-9]*$")
 
 OUTPUT_VAR_NAME = "_opx_out_"
 CONTEXT_VAR_NAME = "_opx_ctx_"
+INCLUDE_VAR_NAME = "_opx_include_"
 
 
 class TemplateCompiler:
@@ -180,7 +181,7 @@ class TemplateCompiler:
 
     def start_else_block(self, start_offset: int, end_offset: int):
         if not hasattr(self.code_modules[-1], "orelse"):
-            self.syntax_error("Cannot start an else block without an if/for block", start_offset, end_offset)
+            raise self.syntax_error("Cannot start an else block without an if/for block", start_offset, end_offset)
 
         self.selected_body = self.code_modules[-1].orelse
 
@@ -196,7 +197,7 @@ class TemplateCompiler:
         return module
 
     def compile_expression(self, expr: str, offset: int) -> None:
-        expression = self._compile_expression(expr[2:-2], offset + 2)
+        expression = self._compile_expression(expr, offset)
         self.append(expression, offset, offset + len(expr))
 
     def _compile_expression(self, expr: str, offset: int) -> ast.expr:
@@ -240,12 +241,12 @@ class TemplateCompiler:
         if len(args) == 1:
             args = self._compile_node(f"[{args[0]}]", start_offset + len(filter_name)).elts
         elif len(args) > 1:
-            self.syntax_error(f"Invalid Syntax", start_offset, end_offset)
+            raise self.syntax_error(f"Invalid Syntax", start_offset, end_offset)
         else:
             args = []
 
         if filter_name not in TEMPLATE_FILTERS:
-            self.syntax_error(f"Unknown filter: {filter_name}", start_offset, start_offset + len(filter_name))
+            raise self.syntax_error(f"Unknown filter: {filter_name}", start_offset, start_offset + len(filter_name))
 
         pipe_name = f"_p_{filter_name}"
         self.variables.define(pipe_name, TEMPLATE_FILTERS[filter_name])
@@ -273,13 +274,13 @@ class TemplateCompiler:
 
     def compile_if(self, expr: str, offset: int):
         if not expr:
-            self.syntax_error("Missing condition expression", offset, offset + len(expr))
+            raise self.syntax_error("Missing condition expression", offset, offset + len(expr))
 
-        expr, start_offset, end_offset = self.str_strip(expr, offset)
+        expr, start_offset, end_offset = self.str_strip(expr, offset + 2)
         n_expr = self._compile_node(expr, start_offset)
         block: ast.If = ast.If(n_expr, body=[], orelse=[])
 
-        self.start_block(block, start_offset, end_offset)
+        self.start_block(block, offset, end_offset)
 
     def compile_for(self, expr: str, offset: int):
         expr, start_offset, end_offset = self.str_strip(expr, offset)
@@ -292,7 +293,7 @@ class TemplateCompiler:
         var_name = var_name.strip()
 
         if not VAR_NAME_RE.match(var_name) or op.strip() != "in" or not container:
-            self.syntax_error("Invalid syntax", start_offset, end_offset)
+            raise self.syntax_error("Invalid syntax", start_offset, end_offset)
 
         self.variables.declare(var_name)
 
@@ -304,11 +305,28 @@ class TemplateCompiler:
         )
         self.start_block(block, start_offset, end_offset)
 
+    def compile_include(self, args: str, tag_offset: int, start_offset: int, end_offset: int) -> None:
+        arg_start_offset = start_offset + 7
+        indent = self.offset_to_position(tag_offset).column - 1
+
+        node = self._compile_node(f"f({args})", arg_start_offset, -2)
+        node.func = ast.Name(id=INCLUDE_VAR_NAME, ctx=ast.Load())
+
+        has_indent = any(k.arg == "indent" for k in node.keywords)
+        node.keywords.append(ast.keyword(arg="context", value=ast.Name(CONTEXT_VAR_NAME, ctx=CTX_LOAD)))
+        if not has_indent:
+            node.keywords.append(ast.keyword(arg="indent", value=ast.Constant(value=indent, kind=None)))
+
+        self.set_location(node, start_offset, end_offset)
+        self.set_location(node.func, start_offset, arg_start_offset)
+        self.append(node, start_offset, end_offset)
+
     def compile_constant(self, const: str, offset: int) -> None:
         self.append(ast.Constant(value=const, kind=None), offset, offset + len(const))
 
-    def _compile_node(self, value: str, offset: int) -> ast.expr:
+    def _compile_node(self, value: str, offset: int, offset_drift: int = 0) -> ast.expr:
         lineno, offset = self.offset_to_position(offset)
+        offset += offset_drift
 
         try:
             expr = ast.parse(value, mode="eval").body
@@ -317,27 +335,36 @@ class TemplateCompiler:
                 ex.offset += offset - 1
 
             ex.lineno += lineno - 1
+            end_pos = None
+
+            # TODO: Remove check after dropping support for Python < 3.10
+            if getattr(ex, "end_lineno", None) is not None:
+                if ex.end_lineno == 1:
+                    ex.end_offset += offset - 1
+
+                ex.end_lineno += lineno - 1
+                end_pos = Position(ex.end_lineno, ex.end_offset)
 
             raise TemplateError(
                 ex.msg,
                 file_name=self.file_path,
                 start_pos=Position(ex.lineno, ex.offset),
-                end_pos=None,
+                end_pos=end_pos,
             ) from None
 
         transformer = TemplateNodeTransformer(self, lineno, offset)
         return transformer.visit(expr)
 
-    def disallowed(self, message: str, start_position: Position, end_position: Position):
-        raise TemplateError(
+    def disallowed(self, message: str, start_position: Position, end_position: Position) -> Exception:
+        return TemplateError(
             message,
             file_name=self.file_path,
-            start_position=start_position,
-            end_position=end_position,
+            start_pos=start_position,
+            end_pos=end_position,
         )
 
-    def syntax_error(self, message: str, start_offset: int, end_offset: int):
-        raise TemplateError(
+    def syntax_error(self, message: str, start_offset: int, end_offset: int) -> Exception:
+        return TemplateError(
             message,
             file_name=self.file_path,
             start_pos=self.offset_to_position(start_offset),
@@ -415,7 +442,7 @@ class TemplateNodeTransformer(ast.NodeTransformer):
         self.update_location(node)
 
         if node.attr.startswith("_"):
-            self.compiler.disallowed(f"Unknown attribute: {node.attr}", *self.node_bounds(node))
+            raise self.compiler.disallowed(f"Unknown attribute: {node.attr}", *self.node_bounds(node))
 
         self.skip_refs = self.generate_ref(node)
         node.value = self.visit(node.value)
@@ -426,7 +453,7 @@ class TemplateNodeTransformer(ast.NodeTransformer):
         self.update_location(node)
 
         if node.id.startswith("_"):
-            self.compiler.disallowed("Variable names cannot start with an underscore", *self.node_bounds(node))
+            raise self.compiler.disallowed("Variable names cannot start with an underscore", *self.node_bounds(node))
 
         self.generate_ref(node)
         if node.id in self.compiler.variables:
